<?php

namespace Drupal\idealyzer;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Database\Database;
use Drupal\Core\Url;
use Drupal\Core\Database\Query\PagerSelectExtender;

/**
 * Defines a class to build a listing of Site Scanner Result entities.
 */
class SiteScannerResultListBuilder extends EntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function render() {

    // Calculate totals.
    $totals = $this->calculateTotals();
    // Create a dynamic title for the accordion.
    $accordion_title = $this->t('@total Results', [
      '@total' => $totals['total'],
    ]);
    
    // Attach the custom CSS library  
    
    $build['#attached']['drupalSettings']['cmsChartData'] = [
      'wordpress' => $totals['cms_wordpress_percentage'],
      'drupal' => $totals['cms_drupal_percentage'],
      'na' => $totals['cms_na_percentage'],
    ];
    
    $build['totals_accordion'] = [
      '#type' => 'details',
      '#title' => $accordion_title,
      '#open' => TRUE,
      '#attributes' => ['class' => ['accordion clearfix']],
      'content' => [
        '#markup' => '<div class="totals-container"><div class="totals-column">' .
                    '<strong>' . $this->t('Unique Agencies:') . '</strong> ' . $totals['unique_agencies'] . '</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('DAP Usage:') . '</strong> ' . $totals['dap_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('HTTP 200 Responses:') . '</strong> ' . $totals['status_200_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('HTTP 403 Responses:') . '</strong> ' . $totals['status_403_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('HTTP 404 Responses:') . '</strong> ' . $totals['status_404_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('IPv6 Enabled:') . '</strong> ' . $totals['ipv6_count'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('CMS WordPress:') . '</strong> ' . $totals['cms_wordpress_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('CMS Drupal:') . '</strong> ' . $totals['cms_drupal_percentage'] . '%</div>' .
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('CMS NA:') . '</strong> ' . $totals['cms_na_percentage'] . '%</div>'.
                    '<div class="totals-column">' .
                    '<strong>' . $this->t('Unique CMS:') . '</strong> ' . $totals['unique_cms_list'] . '</div></div>',
      ],
    ];
   
  $build['totals_accordion']['cms_chart'] = [
    '#type' => 'inline_template',
    '#template' => '<div class="chart-container"><canvas id="cmsChart" width="200" height="200">loading</canvas></div>',
    '#attached' => [
      'library' => [
        'idealyzer/behaviors', // Ensure you have the necessary libraries included.
      ],
    ],
  ];
  $build['#attached']['library'][] = 'idealyzer/styles';
    // Render the table.
    $build['table'] = parent::render();
    \Drupal::logger('idealyzer')->notice(print_r($build, TRUE));
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['scan_date'] = $this->t('Scan Date');
    //$header['target_url_agency_owner'] = $this->t('Agency');
    $header['title'] = $this->t('Title');
    $header['final_url'] = $this->t('Final URL');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['scan_date'] = $entity->get('scan_date')->value;
    //$row['target_url_agency_owner'] = $entity->get('target_url_agency_owner')->value;
    $row['title'] = $entity->get('title')->value;
    $row['final_url'] = [
      'data' => [
        '#type' => 'link',
        '#title' => $entity->get('final_url')->value,
        '#url' => Url::fromUri($entity->get('final_url')->value, ['attributes' => ['target' => '_blank']]),
      ],
    ];
    return $row + parent::buildRow($entity);
  }

  /**
   * Loads the entities with applied filters and pagination.
   *
   * @return array
   *   An array of loaded entities.
   */
  public function load() {
    $query = $this->getFilteredQuery();

    // Add pagination.
    $query = $query->extend(PagerSelectExtender::class)->limit(10);
    $results = $query->execute()->fetchAll();
    $entities = [];

    foreach ($results as $result) {
      $entities[] = $this->storage->load($result->id);
    }

    return $entities;
  }

  /**
   * Gets the filtered query without pagination.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   The filtered query.
   */
  protected function getFilteredQuery() {
    $connection = Database::getConnection();
    $query = $connection->select('site_scanner_results', 'ssr')
      ->fields('ssr', ['id', 'scan_date', 'target_url_agency_owner', 'final_url', 'dap', 'final_url_status_code', 'ipv6', 'cms']); // Add other fields as needed.

    // Apply filters based on query parameters.
    $final_url = \Drupal::request()->query->get('final_url');
    if ($final_url) {
      $query->condition('ssr.final_url', '%' . $connection->escapeLike($final_url) . '%', 'LIKE');
    }

    $keyword = \Drupal::request()->query->get('keyword');
    if ($keyword) {
      $group = $query->orConditionGroup()
        ->condition('ssr.final_url', '%' . $connection->escapeLike($keyword) . '%', 'LIKE')
        ->condition('ssr.target_url_agency_owner', '%' . $connection->escapeLike($keyword) . '%', 'LIKE'); // Adjust fields to be searched.
      $query->condition($group);
    }

    $target_url_domain = \Drupal::request()->query->get('target_url_domain');
    if ($target_url_domain) {
      $query->condition('ssr.target_url_domain', $target_url_domain);
    }

    $agency_owner = \Drupal::request()->query->get('agency_owner');
    if ($agency_owner) {
      $query->condition('ssr.target_url_agency_owner', $agency_owner);
    }

    $final_url_status_code = \Drupal::request()->query->get('final_url_status_code');
    if ($final_url_status_code) {
      $query->condition('ssr.final_url_status_code', $final_url_status_code);
    }

    return $query;
  }

  /**
   * Calculates totals from the filtered list.
   *
   * @return array
   *   An array containing total results, unique agencies, and DAP usage percentage.
   */
  protected function calculateTotals() {
    $query = $this->getFilteredQuery();
    $results = $query->execute()->fetchAll();

    $total_results = count($results);
    $unique_agencies = count(array_unique(array_column($results, 'target_url_agency_owner')));

    // Calculate DAP usage percentage.
    $dap_count = count(array_filter(array_column($results, 'dap'), function($dap) {
      return $dap == 1; // Ensure this is checking for true values.
    }));
    $dap_percentage = $total_results > 0 ? round(($dap_count / $total_results) * 100, 2) : 0;

    // Calculate percentages for HTTP status codes.
    $status_codes = array_column($results, 'final_url_status_code');
    $status_200_count = count(array_filter($status_codes, function($status) {
      return $status == 200;
    }));
    $status_403_count = count(array_filter($status_codes, function($status) {
      return $status == 403;
    }));
    $status_404_count = count(array_filter($status_codes, function($status) {
      return $status == 404;
    }));

    $status_200_percentage = $total_results > 0 ? round(($status_200_count / $total_results) * 100, 2) : 0;
    $status_403_percentage = $total_results > 0 ? round(($status_403_count / $total_results) * 100, 2) : 0;
    $status_404_percentage = $total_results > 0 ? round(($status_404_count / $total_results) * 100, 2) : 0;



    // Calculate CMS percentages.
    $cms_values = array_column($results, 'cms');
    $cms_wordpress_count = count(array_filter($cms_values, function($cms) {
      return strtolower($cms ?? '') == 'wordpress';
    }));
    $cms_drupal_count = count(array_filter($cms_values, function($cms) {
      return strtolower($cms ?? '') == 'drupal';
    }));
    $cms_na_count = $total_results - $cms_wordpress_count - $cms_drupal_count; // Blank or NA

    $cms_wordpress_percentage = $total_results > 0 ? round(($cms_wordpress_count / $total_results) * 100, 2) : 0;
    $cms_drupal_percentage = $total_results > 0 ? round(($cms_drupal_count / $total_results) * 100, 2) : 0;
    $cms_na_percentage = $total_results > 0 ? round(($cms_na_count / $total_results) * 100, 2) : 0;

    // Get unique CMS values.
    $unique_cms_values = array_unique(array_filter($cms_values, function($cms) {
      return !empty($cms);
    }));
    $unique_cms_list = implode(', ', $unique_cms_values);

    // Calculate ipv6 usage percentage.
    $ipv6_count  = count(array_filter(array_column($results, 'ipv6'), function($ipv6 ) {
      return $ipv6 == 1; // Ensure this is checking for true values.
    }));
    $ipv6_count_percentage = $total_results > 0 ? round(($ipv6_count / $total_results) * 100, 2) : 0;

    return [
      'total' => $total_results,
      'unique_agencies' => $unique_agencies,
      'dap_percentage' => $dap_percentage,
      'status_200_percentage' => $status_200_percentage,
      'status_403_percentage' => $status_403_percentage,
      'status_404_percentage' => $status_404_percentage,
      'ipv6_count' => $ipv6_count_percentage,
      'cms_wordpress_percentage' => $cms_wordpress_percentage,
      'cms_drupal_percentage' => $cms_drupal_percentage,
      'cms_na_percentage' => $cms_na_percentage,
      'unique_cms_list' => $unique_cms_list,
    ];
  }



  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = [];

    if ($entity->access('view')) {
      $operations['view'] = [
        'title' => t('View'),
        'weight' => -10,
        'url' => Url::fromRoute('entity.site_scanner_result.canonical', ['site_scanner_result' => $entity->id()]),
      ];
    }

    if ($entity->access('view')) {
      $operations['download_json'] = [
        'title' => t('Download JSON'),
        'weight' => -5,
        'url' => Url::fromRoute('entity.site_scanner_result.download_json', ['site_scanner_result' => $entity->id()]),
      ];
    }

    if ($entity->access('delete')) {
      $operations['delete'] = [
        'title' => t('Delete'),
        'weight' => -1,
        'url' => Url::fromRoute('entity.site_scanner_result.delete_form', ['site_scanner_result' => $entity->id()]),
      ];
    }

    return $operations;
  }
}
